@@ -14,8 +14,10 @@
 #    under the License.
 
 """Volume interface (v3 extension)."""
-from cinderclient.apiclient import base as common_base
+import warnings
+
 from cinderclient import api_versions
+from cinderclient.apiclient import base as common_base
 from cinderclient import base
 from cinderclient.v2 import volumes
 
@@ -49,6 +51,22 @@ class Volume(volumes.Volume):
         """Revert a volume to a snapshot."""
         self.manager.revert_to_snapshot(self, snapshot)
 
+    def migrate_volume(self, host, force_host_copy, lock_volume, cluster=None):
+        """Migrate the volume to a new host."""
+        return self.manager.migrate_volume(self, host, force_host_copy,
+                                           lock_volume, cluster)
+
+    def manage(self, host, ref, name=None, description=None,
+               volume_type=None, availability_zone=None, metadata=None,
+               bootable=False, cluster=None):
+        """Manage an existing volume."""
+        return self.manager.manage(host=host, ref=ref, name=name,
+                                   description=description,
+                                   volume_type=volume_type,
+                                   availability_zone=availability_zone,
+                                   metadata=metadata, bootable=bootable,
+                                   cluster=cluster)
+
 
 class VolumeManager(volumes.VolumeManager):
     resource_class = Volume
@@ -59,7 +77,7 @@ class VolumeManager(volumes.VolumeManager):
                volume_type=None, user_id=None,
                project_id=None, availability_zone=None,
                metadata=None, imageRef=None, scheduler_hints=None,
-               source_replica=None, multiattach=False):
+               source_replica=None, multiattach=False, backup_id=None):
         """Create a volume.
 
         :param size: Size of volume in GB
@@ -79,7 +97,8 @@ class VolumeManager(volumes.VolumeManager):
         :param scheduler_hints: (optional extension) arbitrary key-value pairs
                             specified by the client to help boot an instance
         :param multiattach: Allow the volume to be attached to more than
-                            one instance
+                            one instance (deprecated)
+        :param backup_id: ID of the backup
         :rtype: :class:`Volume`
         """
         if metadata is None:
@@ -87,6 +106,13 @@ class VolumeManager(volumes.VolumeManager):
         else:
             volume_metadata = metadata
 
+        if multiattach:
+            warnings.warn('The ``multiattach`` volume create flag is '
+                          'deprecated and will be removed in a future '
+                          'release. Multiattach capability is now controlled '
+                          'using volume type extra specs.',
+                          DeprecationWarning)
+
         body = {'volume': {'size': size,
                            'consistencygroup_id': consistencygroup_id,
                            'snapshot_id': snapshot_id,
@@ -103,6 +129,7 @@ class VolumeManager(volumes.VolumeManager):
                            'source_volid': source_volid,
                            'source_replica': source_replica,
                            'multiattach': multiattach,
+                           'backup_id': backup_id
                            }}
 
         if group_id:
@@ -147,7 +174,7 @@ class VolumeManager(volumes.VolumeManager):
 
         return common_base.ListWithMeta([], response_list)
 
-    @api_versions.wraps("3.15")
+    @api_versions.wraps("3.15")  # noqa: F811
     def delete_metadata(self, volume, keys):
         """Delete specified keys from volumes metadata.
 
@@ -178,7 +205,7 @@ class VolumeManager(volumes.VolumeManager):
                              'container_format': container_format,
                              'disk_format': disk_format})
 
-    @api_versions.wraps("3.1")
+    @api_versions.wraps("3.1")  # noqa: F811
     def upload_to_image(self, volume, force, image_name, container_format,
                         disk_format, visibility, protected):
         """Upload volume to image service as image.
@@ -194,6 +221,46 @@ class VolumeManager(volumes.VolumeManager):
                              'visibility': visibility,
                              'protected': protected})
 
+    def migrate_volume(self, volume, host, force_host_copy, lock_volume,
+                       cluster=None):
+        """Migrate volume to new backend.
+
+        The new backend is defined by the host or the cluster (not both).
+
+        :param volume: The :class:`Volume` to migrate
+        :param host: The destination host
+        :param force_host_copy: Skip driver optimizations
+        :param lock_volume: Lock the volume and guarantee the migration
+                            to finish
+        :param cluster: The cluster
+        """
+        body = {'host': host, 'force_host_copy': force_host_copy,
+                'lock_volume': lock_volume}
+
+        if self.api_version.matches('3.16'):
+            if cluster:
+                body['cluster'] = cluster
+                del body['host']
+
+        return self._action('os-migrate_volume', volume, body)
+
+    def manage(self, host, ref, name=None, description=None,
+               volume_type=None, availability_zone=None, metadata=None,
+               bootable=False, cluster=None):
+        """Manage an existing volume."""
+        body = {'volume': {'host': host,
+                           'ref': ref,
+                           'name': name,
+                           'description': description,
+                           'volume_type': volume_type,
+                           'availability_zone': availability_zone,
+                           'metadata': metadata,
+                           'bootable': bootable
+                           }}
+        if self.api_version.matches('3.16') and cluster:
+            body['cluster'] = cluster
+        return self._create('/os-volume-manage', body, 'volume')
+
     @api_versions.wraps("3.8")
     def list_manageable(self, host, detailed=True, marker=None, limit=None,
                         offset=None, sort=None):
@@ -211,7 +278,7 @@ class VolumeManager(volumes.VolumeManager):
 
         return self._get('/scheduler-stats/get_pools%s' % query_string, None)
 
-    @api_versions.wraps("3.33")
+    @api_versions.wraps("3.33")  # noqa: F811
     def get_pools(self, detail, search_opts):
         """Show pool information for backends."""
         # pylint: disable=function-redefined
